# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_delay.m
# input: AlphaU upper arrival bound
#        BetaL lower servcie curve
#        BetaU upper service curve
#       
# output PhiU upper delay curve
#
# description: The function computes the
#              upper delay curve for a system.
#
###########################################
function PhiU = sla_delay(AlphaU, BetaL, BetaU)

   if (sla_is_increasing(AlphaU) && sla_is_increasing(BetaL) && sla_is_increasing(BetaU) && sla_is_smaller_or_equal(BetaL, BetaU))
      GammaU = sla_min(sla_min_conv(sla_min_deconv(AlphaU, BetaL),BetaL),BetaL) ;
      GammaUI = sla_invert(GammaU) ;
      AlphaUI = sla_invert(AlphaU) ;
      XX = sla_sub(GammaUI, AlphaUI) ;
      # to obtain teh accumulated delay, we have to integrate XX
      # which would result in a quadratic function, we use a linear 
      # over-approximation instead
      # this becomes exact for piecewise constant delays
      x0 = XX{1}(1) ;
      y0 = 0 ;
      if (XX{1}(3) == 0)
      	 s0 = XX{1}(2) ;
      else 
         if (columns(XX) == 1)
	    # this gives an infinite increase
	    s0 = 1.0e+12 ;
	 else
	    s0 = XX{1}(2) + 0.5*XX{1}(3)*(XX{2}(1)-XX{1}(1)) ;
	 endif
      endif
      PhiU = {[x0, y0, s0]} ;
      for (i=2:columns(XX))
         x0 = XX{i}(1) ;
	 y0 = (x0 - PhiU{i-1}(1))*PhiU{i-1}(3)+PhiU{i-1}(2) ;
      	 if (XX{i}(3) == 0)
      	    s0 = XX{i}(2) ;
         else 
            if (columns(XX) == i)
	       # this gives an infinite increase
	       s0 = 1.0e+12 ;
	    else
	       s0 = XX{i}(2) + 0.5*XX{i}(3)*(XX{i+1}(1)-XX{i}(1)) ;
	    endif
         endif
	 PhiU{i}= [x0, y0, s0] ;
      endfor
      PhiU = sla_reduce(PhiU) ;
   else
      printf("Curves are inconsistent\nSTOP COMPUTATION\n") ;
   endif
   PhiU = sla_reduce(PhiU) ;
endfunction