# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_input_delay.m
# input: Phi1L lower cummulated delay bound service 1
#        Phi1U upper cummulated delay bound service 1
#        Alpha1U upper cummulated arrival bound service 1
#        Alpha2U upper cummulated arrival bound service 2
#       
# output AlphaU delayed input bound
#
# description: The function computes a delayed arrival curve
#              for the first service such that the output curve of
#              the first service is conform to the upper arrival curve of the
#              second service 
#
###########################################
function AlphaU = sla_input_delay(Phi1L, Phi1U, Alpha1U, Alpha2U)

   # first compute t^*
   [GammaU, GammaUI] = sla_output_bound(Phi1L, Alpha1U) ;
   [GammaB, GammaBI] = sla_output_bound(Phi1U, Alpha1U) ;
   tval = GammaB{2}(1) ;
   for (i=1:columns(GammaU))
       if (GammaU{i}(1) >= tval)
          i-- ;
	  break ;
       endif
   endfor
   xstart = (tval-GammaU{i}(1))*GammaU{i}(3)+GammaU{i}(2) ;
   phid = sla_derivative(Phi1L) ;
   Alpha2UI = sla_invert(Alpha2U) ;
   i1 = 1 ;
   i2 = 1 ;
   AlphaUI = {[0,0,0]} ;
   j = 2 ;
   # Find the xstart of Alpha2UI 
   while (i1 < columns(Alpha2UI))
      if (Alpha2UI{i1}(1) < xstart) 
         i1++ ;
      endif
   endwhile
   # Compute the point where the cuurve becomes non-zero
   xcurrent = xstart ;
   while (true)
      xval = xcurrent - xstart ;
      yval = Alpha2UI{i1}(2) + (xcurrent-Alpha2UI{i1}(1)) * Alpha2UI{i1}(3) ;
      pval = pval = phid{i2}(2) ;
      if (yval >= pval)
         yval = yval - pval ;
	 sval = Alpha2UI{i1}(3) ;
	 break ;
      else
	 xnext = (pval-Alpha2UI{i1}(2))/Alpha2UI{i1}(3) + Alpha2UI{i1}(1) ;
         if (i1 < columns(Alpha2UI) && i2 < columns(phid))
	    if (Alpha2UI{i1+1}(1) - xstart < phid{i2+1}(1))
	       if (xnext < Alpha2UI{i1+1}(1))
	          xval = xnext - xstart ;
		  yval = 0 ;
		  sval = Alpha2UI{i1}(3) ;
 		  break ;
	       else
	          i1++ ;
		  xcurrent = Alpha2UI{i1}(1) ;
	       endif
	    else
	       if (xnext-xstart < phid{i2+1}(1))
	          xval = xnext - xstart ;
		  yval = 0 ;
		  sval = Alpha2UI{i1}(3) ;
 		  break ;
	       else
	          i2++ ;
		  xcurrent = phid{i2}(1) + xstart ;
	       endif
	    endif
	 else
	    if (i1 < columns(Alpha2UI))
	       if (xnext < Alpha2UI{i1+1}(1))
	          xval = xnext - xstart ;
		  yval = 0 ;
		  sval = Alpha2UI{i1}(3) ;
 		  break ;
	       else
	          i1++ ;
		  xcurrent = Alpha2UI{i1}(1) ;
	       endif
	    else
	       if (i2 < columns(phid))
	          if (xnext-xstart < phid{i2+1}(1))
	             xval = xnext - xstart ;
		     yval = 0 ;
		     sval = Alpha2UI{i1}(3) ;
 		     break ;
	          else
	             i2++ ;
		     xcurrent = phid{i2}(1) + xstart ;
	          endif
	       else
	          xval = xnext - xstart ;
		  yval = 0 ;
		  sval = Alpha2UI{i1}(3) ;
 		  break ;
	       endif
	    endif
	 endif
      endif
   endwhile
   AlphaUI{j} = [xval, yval, sval] ;
   j++ ;
   while (i1 < columns(Alpha2UI) || i2 < columns(phid))
      if (i1 < columns(Alpha2UI) && i2 < columns(phid))
         if (Alpha2UI{i1+1}(1) - xstart < phid{i2+1}(1))
	    i1++ ;
	    xval = Alpha2UI{i1}(1) - xstart ;
	    yval = max(Alpha2UI{i1}(2)-pval, 0) ;
   	    sval = Alpha2UI{i1}(3) ;
	 else
	    if (Alpha2UI{i1+1}(1) - xstart > phid{i2+1}(1))
	       i2++ ;
	       pval = phid{i2}(2) ;
	       xval = phid{i2}(1) ;
	       yval = max(Alpha2UI{i1}(2)-pval, 0) ;
   	       sval = Alpha2UI{i1}(3) ;
	    else
	       i1++ ;
	       i2++ ;
	       pval = phid{i2}(2) ;
	       xval = Alpha2UI{i1}(1) - xstart ;
	       yval = max(Alpha2UI{i1}(2)-pval, 0) ;
   	       sval = Alpha2UI{i1}(3) ;
	    endif
	 endif
      else
         if (i1 < columns(Alpha2UI))
	    i1++ ;
	    xval = Alpha2UI{i1}(1) - xstart ;
	    yval = max(Alpha2UI{i1}(2)-pval, 0) ;
   	    sval = Alpha2UI{i1}(3) ;
	 else
	    i2++ ;
	    pval = phid{i2}(2) ;
	    xval = phid{i2}(1) ;
	    yval = max(Alpha2UI{i1}(2)-pval, 0) ;
   	    sval = Alpha2UI{i1}(3) ;
	 endif
      endif
      AlphaUI{j} = [xval, yval, sval] ;
      j++ ;
   endwhile
   AlphaUI = sla_reduce(AlphaUI) ;
   AlphaU = sla_invert(AlphaUI) ;
   AlphaU = sla_reduce(AlphaU) ;

endfunction