# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_lower_delay.m
# input: AlphaU upper arrival bound
#        SigmaL lower servcie curve
#        SigmaU upper service curve
#       
# output PhiL lower delay bound
#        PhiU upper delay curve
#
# description: The function computes the
#              lower delay curve for a system.
#
###########################################
function PhiL = sla_lower_delay(AlphaU, SigmaL, SigmaU)

   if (sla_is_increasing(AlphaU) && sla_is_increasing(SigmaL) &&  sla_is_increasing(SigmaU) && sla_is_smaller_or_equal(SigmaL, SigmaU))
      # The case AlphaU has to be handled specifically. In this case teh lower
      # delay is zero.
      if (sla_is_zero(AlphaU))
        PhiL = {[0,0,0]} ;
        return ;
      end
      GammaU = sla_min(sla_min_deconv(sla_min_conv(AlphaU, SigmaU),SigmaU),SigmaU) ;
      GammaUI = sla_invert(GammaU) ;
      AlphaUI = sla_invert(AlphaU) ;
      XX = sla_sub(sla_max(GammaUI, AlphaUI), AlphaUI) ;
      # to obtain the accumulated delay, we have to integrate XX or YY
      # which would result in a quadratic function, we use a linear 
      # approximation instead
      # this becomes exact for piecewise constant delays
      # we start with the lower delay
      x0 = XX{1}(1) ;
      y0 = 0 ;
      if (XX{1}(3) == 0)
      	 s0 = XX{1}(2) ;
      else 
         if (columns(XX) == 1)
	    # this gives an infinite increase
	    s0 = inf ;
	 else
	    s0 = XX{1}(2) + 0.5*XX{1}(3)*(XX{2}(1)-XX{1}(1)) ;
	 endif
      endif
      PhiL = {[x0, y0, s0]} ;
      for (i=2:columns(XX))
         x0 = XX{i}(1) ;
	 y0 = (x0 - PhiL{i-1}(1))*PhiL{i-1}(3)+PhiL{i-1}(2) ;
      	 if (XX{i}(3) == 0)
      	    s0 = XX{i}(2) ;
         else 
            if (columns(XX) == i)
	       # this gives an infinite increase
	       s0 = inf ;
	    else
	       s0 = XX{i}(2) + 0.5*XX{i}(3)*(XX{i+1}(1)-XX{i}(1)) ;
	    endif
         endif
	 PhiL{i}= [x0, y0, s0] ;
      endfor
   else
      printf("Curves are inconsistent in sla_lower_delay\nSTOP COMPUTATION\n") ;
   endif
   PhiL = sla_reduce(PhiL) ;
   # PhiL = sla_convex_lower(PhiL) ;
endfunction