# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_min_deconv
#
# input: A, B matrices of the type
#        A {a1 a2 an}
#        where ai = [xi, yi, si]
#
# output C = min_deconvolution(A, B)
#        C = {[x, inf, inf]} shows that
#           the function is \infty from x on.
#
#  Description:
#  Computation componentwise for segements
#  ai = y1 + s1(t-a)  and t \in [a,b)
#  bi = y2 + s2(t-c)  and t \in [c,d)
#  if s1 >= s2
#  case a:
#    b = d = infty => curve is infinite
#  case b:
#    t in [(a-d)^+, (b-d)) and b-d>0:
#    y1 - y2 - (d-c)s2 + (t-a+d)s1
#  case c:
#    t in [(b-d)^+, (b-c)) and b-c > 0, b < infty:
#    y1 - y2 + (b-a)s1 + (t-b+c)s2
#  if s1 < s2:
#  case d:
#    t in [(a-d)^+, (a-c)) and a-c > 0:
#    y1 - y2 - (t-a+c)s2
#  case e:
#    t in [(a-c)^+, (b-c)) and b-c > 0
#    y1 - y2 + (t-a+c)s1
#    
#
#  case f: in all remaining cases the curve is empty  
# 
#
###########################################
function C = sla_min_deconv(A, B)

   # define a lower bound for the convolution
   # since it is implicitly assumed that all curves have a spot 0
   # in the origion we can use the first curve as lower bound
   # for the deconvolution
   C = A ;
   [xC, yC] = sla_curve_to_xy(C{columns(C)}(1)+1, C) ;
   # define large and small values as substitutes for +/-infinity
   large_value = inf ;
   small_value = min(min(xC),min(yC)) ;
   # concatenate all segements and build the maxima
   for (i=1:columns(A))
      for (j=1:columns(B))
      	 # we distinguish according to the slope in both segements
	 if (A{i}(3) > B{j}(3))
	    # case a both segements are infinite
	    if (i == columns(A) && j == columns(B))
	       # printf("Curve is infinite for all t in R_+\n") ;
               C = {[0, large_value, large_value]} ;
	       return ; 
	    endif
	    # case b
	    if (j < columns(B))
	       if (i == columns(A) || A{i+1}(1) > B{j+1}(1))
	          x1 = A{i}(1)-B{j+1}(1) ;
	          if (x1 < 0)
	             x1 = 0 ;
	          endif
	          y1 = A{i}(2) - B{j}(2) - (B{j+1}(1)-B{j}(1))*B{j}(3) + (x1-(A{i}(1)-B{j+1}(1)))*A{i}(3);
 	          if (i == columns(A))
	             x2 = x1 + 1 ;
		     y2 = y1 + A{i}(3) ;
	             xx = [x1, x2] ;
	             yy = [y1, y2] ;
	             [xC, yC] = sla_max_segment(xC, yC, false, xx, yy, false) ;
# printf("Case b1 i %i j %i\n", i, j) ;
# xx
# yy
	          else
	             x2 = A{i+1}(1) - B{j+1}(1) ;
		     y2 = y1 + (x2 -x1)*A{i}(3) ;
	             xx = [x1, x2] ;
	             yy = [y1, y2] ;
	             [xC, yC] = sla_max_segment(xC, yC, false, xx, yy, true) ;
# printf("Case b2 i %i j %i\n", i, j) ;
# xx
# yy
		  endif
	       endif
	    endif
	    # case c
  	    if (i < columns(A) && A{i+1}(1) > B{j}(1))
	       if (j == columns(B))
	          x1 = 0 ;
	       else
	          x1 = A{i+1}(1) - B{j+1}(1) ;
		  if (x1 < 0)
		     x1 = 0 ;
		  endif
	       endif
	       y1 = A{i}(2) - B{j}(2) + (A{i+1}(1)-A{i}(1))*A{i}(3) + (x1-(A{i+1}(1)-B{j}(1)))*B{j}(3) ;
	       x2 = A{i+1}(1)-B{j}(1) ;
	       y2 = y1 + (x2 - x1)*B{j}(3) ;
	       xx = [x1, x2] ;
	       yy = [y1, y2] ;
	       [xC, yC] = sla_max_segment(xC, yC, false, xx, yy, true) ;
# printf("Case C \n") ;
# xx
# yy
	    endif
         else
	    # case d:
	    if (A{i}(1) > B{j}(1))
	       if (j == columns(B))
	          x1 = 0 ;
	       else
	          x1 = A{i}(1)-B{j+1}(1) ;
	          if (x1 < 0)
	             x1 = 0 ;
	          endif
	       endif
	       y1 = A{i}(2) - B{j}(2) + (x1-A{i}(1)+B{j}(1))*B{j}(3) ;
 	       x2 = A{i}(1)-B{j}(1) ;
	       y2 = A{i}(2) - B{j}(2) ;
	       xx = [x1, x2] ;
	       yy = [y1, y2] ;
	       [xC, yC] = sla_max_segment(xC, yC, false, xx, yy, true) ;
# printf("Case d i %i j %i\n", i, j) ;
# xx
# yy
            endif
	    # case e:
	    if (i == columns(A) || A{i+1}(1) > B{j}(1))
	       x1 = A{i}(1) - B{j}(1) ;
	       if (x1 < 0)
	          x1 = 0 ;
	       endif
	       y1 = A{i}(2) - B{j}(2) + (x1 - A{i}(1) + B{j}(1))*A{i}(3) ;
	       if (i == columns(A))
	          x2 = x1 + 1 ;
		  y2 = y1 + A{i}(3) ;
	          xx = [x1, x2] ;
	          yy = [y1, y2] ;
	          [xC, yC] = sla_max_segment(xC, yC, false, xx, yy, false) ;
# printf("Case e1 i %i j %i\n", i, j) ;
# xx
# yy
	       else
	          x2 = A{i+1}(1) - B{j}(1) ;
		  y2 = y1 + (x2-x1)*A{i}(3) ;
	          xx = [x1, x2] ;
	          yy = [y1, y2] ;
	          [xC, yC] = sla_max_segment(xC, yC, false, xx, yy, true) ;
# printf("Case e2 i %i j %i\n", i, j) ;
# xx
# yy
	       endif
	    endif
	 endif
      endfor 
   endfor
   C = sla_xy_to_curve(xC, yC) ;
   C = sla_reduce(C) ;

endfunction