# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_parallel_or.m
# input: Alpha1L, Alpha1U input bounds service 1
#        Phi1L, Phi1U delay bounds service 1
#        Alpha2L, Alpha2U input bounds service 2
#        Phi2L, Phi2U delay bounds service 2
#	 b maximal size of load unit
#        p1 probability of choosing 1
#          
# output AlphaL, AlphaU arrival bounds
#        PhiL, PhiU delay bounds 
#
# description: The function computes parameters
#              of a parallel and composition of servcies.
#
###########################################
function [AlphaL, AlphaU, PhiL, PhiU] = sla_parallel_or(Alpha1L, Alpha1U, Phi1L, Phi1U, Alpha2L, Alpha2U, Phi2L, Phi2U, bmax, p1)

   if ((p1 < 0) || (p1 > 1) || (bmax < 0))
      printf("Wrong parameters in sla_parallel_or\n") ;
      return ;
   endif
   # We start with the arrival bounds of the composed service
   p2 = 1-p1 ;
   if (p1 > 0 && not(sla_is_zero(Alpha1L)))
      Alpha1L = sla_add_constant(Alpha1L, p2*bmax, false) ;
      Alpha1L = sla_mult_constant(Alpha1L, 1.0/p1) ;
   else
      Alpha1L = {[0,0,0]} ;
   endif
   if (p2 > 0 && not(sla_is_zero(Alpha2L)))
     Alpha2L = sla_add_constant(Alpha2L, p1*bmax, false) ;
     Alpha2L = sla_mult_constant(Alpha2L, 1.0/p2, false) ;
   else
     Alpha2L = {[0,0,0]} ;
   endif
   AlphaL = sla_max(Alpha1L, Alpha2L) ;
   if (p1 > 0)
     Alpha1U = sla_mult_constant(Alpha1U, 1.0/p1) ;
     Alpha1U = sla_add_constant(Alpha1U, -bmax, true) ;
   else
     Alpha1U = {[0,inf,inf]} ;
   endif
   if (p2 > 0)
     Alpha2U = sla_mult_constant(Alpha2U, 1.0/p2) ;
     Alpha2U = sla_add_constant(Alpha2U, -bmax, true) ;
   else
     Alpha2U = {[0,inf,inf]} ;
   endif
   AlphaU = sla_min(Alpha1U, Alpha2U) ;
   # AlphaL cannot be larger than AlphaU
   if (not(sla_is_smaller_or_equal(AlphaL, AlphaU)))
     printf("AlphaL exceeds AlphaU, set lower bound to maximum\n") ;
     AlphaL = sla_min(AlphaL, AlphaU) ;
   endif
   # Compute the delay bounds
   # First the upper delay bound
   Phi1U = sla_upper_or_delay(Phi1U, bmax, p1) ;
   Phi2U = sla_upper_or_delay(Phi2U, bmax, p2) ;
   PhiU = sla_max(Phi1U, Phi2U) ;
   # Then the lower delay bound
   Phi1L = sla_lower_or_delay(Phi1L, bmax, p1) ;
   Phi2L = sla_lower_or_delay(Phi2L, bmax, p2) ;
   PhiL = sla_min(Phi1L, Phi2L) ;
   # reduce, if possible, the represnetations
   AlphaL = sla_reduce(AlphaL) ;
   AlphaU = sla_reduce(AlphaU) ;
   PhiL = sla_reduce(PhiL) ;
   PhiU = sla_reduce(PhiU) ;

endfunction