# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_tranform_arrival_curves.m
# input: AL, AU  matrices of the type
#        {a1 a2 an}
#        where ai = [xi, yi, si]
#       
# output A1L, A1U transformed arrival curves that
#                 are conform with the requirements
#        status = 2 everything ok, nothing has been modified
#                 1 lower or upper bound have been substitued by closure operations
#                 0 lower or upper bound has been modified to be consistent
#                -1 serious error, curves should not be used
#
# description:
# The function transforms a pair of arrival curves in such a way that
# the lower bound is smaller than the upper bound
# and the upper bound is sub-additive, the lower bound supper-additive. 
#
###########################################
function [A1L, A1U, status] = sla_transform_arrival_curves(AL, AU)

  status = 2 ;
  AL = sla_add_zero(AL) ;
  if (!(sla_check_curve(AL)) || !(sla_is_increasing(AL)) || AL{1}(2) > 0)
    printf("Lower curve is not increasing or includes negative parts, substitute by generic curve\n") ;
    A1L = {[0,0,0]} ;
    status = -1 ;
  else
    A1L = AL ;
  endif
  AU = sla_add_zero(AU) ;
  if (!(sla_check_curve(AU)) || !(sla_is_increasing(AU)) || AU{1}(2) <= 0)
    printf("Upper curve is not increasing or includes negative parts, substitute by generic curve\n") ;
    A1U = {[0,0,0]} ;
    status = -1 ;
  else
    A1U = AU ;
  endif
  # compute the subadditive closure
  [A1L, stat] = sla_max_closure(AL)  ;
  if (stat < status)
    status = stat ;
  endif
  [A1U, stat] = sla_min_closure(AU) ;
  if (stat < status)
    status = stat ;
  endif
  # causality has to be checked as in real time calculus
  A2L = sla_min_conv(A1U, A1L) ;
  if (!sla_is_equal(A1L, A2L))
    A1L = sla_convex_lower(A2L) ;
  endif
  A2U = sla_max_conv(A1L, A1U) ;
  if (!sla_is_equal(A1U, A2U))
    A1U = sla_concave_upper(A2U) ;
  endif
  
endfunction