# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#####################################
# Copyright (c) 2016 Peter Buchholz #
#####################################
############################################
#
# sla_upper_delay.m
# input: AlphaL lower arrival bound
#        SigmaL lower servcie curve
#        SigmaU upper service curve
#       
# output PhiU upper delay curve
#
# description: The function computes the
#              upper delay curve for a system.
#
###########################################
function PhiU = sla_upper_delay(AlphaL, SigmaL, SigmaU)

   if (sla_is_increasing(AlphaL) && sla_is_increasing(SigmaL) && sla_is_increasing(SigmaU) && sla_is_smaller_or_equal(SigmaL, SigmaU))
      GammaL  = sla_min(sla_min(sla_min_conv(sla_min_deconv(AlphaL, SigmaU),SigmaL),SigmaL), AlphaL) ;
      GammaLI = sla_invert(GammaL) ;
      AlphaLI = sla_invert(AlphaL) ;
      # the delay due to backlogs has to be added
      # BU = sla_sub(AlphaL, SigmaL) ;
      # bmax = BU{columns(BU)}(2) ;
      # GammaLI = sla_shift_curve(GammaLI, bmax) ;
      YY = sla_sub(sla_max(GammaLI, AlphaLI), AlphaLI) ;
      # to obtain the accumulated delay, we have to integrate XX or YY
      # which would result in a quadratic function, we use a linear 
      # approximation instead
      # this becomes exact for piecewise constant delays
      if (sla_test_upper_limit(GammaLI, AlphaLI))
      	# we first compute the difference between departure and arrival stream
        x0 = YY{1}(1) ;
        y0 = 0 ;
        if (YY{1}(3) == 0)
          s0 =YY{1}(2) ;
        else 
          if (columns(YY) == 1)
	     # this gives an infinite increase
	    s0 = inf ;
	  else
	     s0 = YY{1}(2) + 0.5*YY{1}(3)*(YY{2}(1)-YY{1}(1)) ;
	  endif
        endif
        PhiU = {[x0, y0, s0]} ;
        for (i=2:columns(YY))
          x0 = YY{i}(1) ;
	  y0 = (x0 - PhiU{i-1}(1))*PhiU{i-1}(3)+PhiU{i-1}(2) ;
      	  if (YY{i}(3) <= 1.0e-12)
      	     s0 = YY{i}(2) ;
          else 
             if (columns(YY) == i)
	        # this gives an infinite increase
	        s0 = inf ;
	     else
	        s0 = YY{i}(2) + 0.5*YY{i}(3)*(YY{i+1}(1)-YY{i}(1)) ;
	     endif
          endif
	  PhiU{i}= [x0, y0, s0] ;
       endfor
     else
       # the arrival stream ecxceeds the departure stream which results in an infinite delay
       PhiU = {[0,inf, inf]} ;
     endif
   else
      printf("Curves are inconsistent in sla_upper_delay\nSTOP COMPUTATION\n") ;
   endif
   PhiU = sla_reduce(PhiU) ;
   PhiU = sla_concave_upper(PhiU) ;
endfunction